/*! @file utils.h
	
	@brief lXȊ֐
 
	@author Makoto Fujisawa
	@date 2008
*/

#ifndef _RX_UTILS_H_
#define _RX_UTILS_H_

#pragma warning (disable: 4101)

//-----------------------------------------------------------------------------
// CN[ht@C
//-----------------------------------------------------------------------------
// CW
#include <ctime>
#include <cmath>
#include <cctype>
#include <cstdio>
#include <cstdlib>

#include <iostream>
#include <fstream>
#include <sstream>

// STL
#include <vector>
#include <string>
#include <deque>
#include <map>
#include <queue>
#include <bitset>

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "rx_vec.h"		// Vec3

#include <ode/ode.h>

using namespace std;

//-----------------------------------------------------------------------------
// 萔
//-----------------------------------------------------------------------------
#define RX_MAX_CONTACTS 1024

const float RX_FEQ_EPS = (float)(1.0e-10);

// deg->rad ̕ϊ(pi/180.0)
const float RX_DEGREES_TO_RADIANS = (float)0.0174532925199432957692369076848;
// rad->deg ̕ϊ(180.0/pi)
const float RX_RADIANS_TO_DEGREES = (float)57.295779513082320876798154814114;

template<class T> 
inline T RX_TO_RADIANS(const T &x){ return static_cast<T>((x)*RX_DEGREES_TO_RADIANS); }
template<class T> 
inline T RX_TO_DEGREES(const T &x){ return static_cast<T>((x)*RX_RADIANS_TO_DEGREES); }


//-----------------------------------------------------------------------------
// lXȊ֐
//-----------------------------------------------------------------------------
/*!
 * lXȌ^stringւ̕ϊ(stringstreamgp)
 * @param[in] x 
 * @return string^ɕϊ
 */
template<typename T>
inline string ToString(const T &x)
{
	stringstream ss;
	ss << x;
	return ss.str();
}


//-----------------------------------------------------------------------------
// MARK:ODE
//-----------------------------------------------------------------------------
inline Vec3 ODE_GetPosition(dBodyID body)
{
	Vec3 pos;
	pos[0] = dBodyGetPosition(body)[0];
	pos[1] = dBodyGetPosition(body)[1];
	pos[2] = dBodyGetPosition(body)[2];
	return pos;
}
inline Vec3 ODE_GetPositionG(dGeomID geom)
{
	Vec3 pos;
	pos[0] = dGeomGetPosition(geom)[0];
	pos[1] = dGeomGetPosition(geom)[1];
	pos[2] = dGeomGetPosition(geom)[2];
	return pos;
}


inline void ODE_GetRotationMatrix(dBodyID body, GLfloat m[16])
{
	m[0]  = (GLfloat)dBodyGetRotation(body)[0];
	m[1]  = (GLfloat)dBodyGetRotation(body)[4]; 
	m[2]  = (GLfloat)dBodyGetRotation(body)[8]; 
	m[3]  = 0.0f; 

	m[4]  = (GLfloat)dBodyGetRotation(body)[1]; 
	m[5]  = (GLfloat)dBodyGetRotation(body)[5]; 
	m[6]  = (GLfloat)dBodyGetRotation(body)[9]; 
	m[7]  = 0.0f; 

	m[8]  = (GLfloat)dBodyGetRotation(body)[2]; 
	m[9]  = (GLfloat)dBodyGetRotation(body)[6]; 
	m[10] = (GLfloat)dBodyGetRotation(body)[10]; 
	m[11] = 0.0f; 

	m[12] = (GLfloat)dBodyGetRotation(body)[3]; 
	m[13] = (GLfloat)dBodyGetRotation(body)[7]; 
	m[14] = (GLfloat)dBodyGetRotation(body)[11]; 
	m[15] = 1.0f;
}

inline Vec3 ODE_GetLinearVel(dBodyID body)
{
	Vec3 vel;
	vel[0] = dBodyGetLinearVel(body)[0];
	vel[1] = dBodyGetLinearVel(body)[1];
	vel[2] = dBodyGetLinearVel(body)[2];
	return vel;
}

inline Vec3 ODE_GetAngularVel(dBodyID body)
{
	Vec3 vel;
	vel[0] = dBodyGetAngularVel(body)[0];
	vel[1] = dBodyGetAngularVel(body)[1];
	vel[2] = dBodyGetAngularVel(body)[2];
	return vel;
}



template<class T>
inline void ODE_Matrix3ToM16(T m[16], const dMatrix3 m3)
{
	m[0]  = (T)m3[0];
	m[1]  = (T)m3[4]; 
	m[2]  = (T)m3[8]; 
	m[3]  = (T)0.0;

	m[4]  = (T)m3[1];
	m[5]  = (T)m3[5];
	m[6]  = (T)m3[9];
	m[7]  = (T)0.0;

	m[8]  = (T)m3[2];
	m[9]  = (T)m3[6];
	m[10] = (T)m3[10];
	m[11] = (T)0.0;

	m[12] = (T)m3[3];
	m[13] = (T)m3[7];
	m[14] = (T)m3[11];
	m[15] = (T)1.0;
}

template<class T>
inline void ODE_QToM16(T m[16], const dQuaternion q)
{
	dReal qq1 = 2*q[1]*q[1];
	dReal qq2 = 2*q[2]*q[2];
	dReal qq3 = 2*q[3]*q[3];

	m[0]  = (T)(1-qq2-qq3);
	m[1]  = (T)(2*(q[1]*q[2]+q[0]*q[3])); 
	m[2]  = (T)(2*(q[1]*q[3]-q[0]*q[2])); 
	m[3]  = (T)(0.0);

	m[4]  = (T)(2*(q[1]*q[2]-q[0]*q[3]));
	m[5]  = (T)(1-qq1-qq3);
	m[6]  = (T)(2*(q[2]*q[3]+q[0]*q[1]));
	m[7]  = (T)(0.0);

	m[8]  = (T)(2*(q[1]*q[3]+q[0]*q[2]));
	m[9]  = (T)(2*(q[2]*q[3]-q[0]*q[1]));
	m[10] = (T)(1-qq1-qq2);
	m[11] = (T)(0.0);

	m[12] = (T)(0.0);
	m[13] = (T)(0.0);
	m[14] = (T)(0.0);
	m[15] = (T)(1.0);
}

//-----------------------------------------------------------------------------
// MARK:s񉉎Z
//-----------------------------------------------------------------------------
template<class T>
static void MultiMatrix4x4(T m[16], const T a[16], const T b[16])
{
	for(int i = 0; i < 4; ++i){
		for(int j = 0; j < 4; ++j){
			m[4*i+j] = 0.0;
			for(int c = 0; c < 4; ++c){
				m[4*i+j] += a[4*i+c]*b[4*c+j];
			}
		}
	}
}

template<class T>
static Vec3 MultiMatrixVector3(T m[16], const Vec3 &v)
{
	double w = (v[0]*m[3]+v[1]*m[7]+v[2]*m[11]+m[15]);
    
	if(w != 0.0) return Vec3(0.0);

	Vec3 dst;
	dst[0] = (v[0]*m[0]+v[1]*m[4]+v[2]*m[8]+m[12])/w;
	dst[1] = (v[0]*m[1]+v[1]*m[5]+v[2]*m[9]+m[13])/w;
	dst[2] = (v[0]*m[2]+v[1]*m[6]+v[2]*m[10]+m[14])/w;

	return dst;
}



template<class T>
static void MakeIdentity(T m[16])
{
	m[0]  = (T)(1.0);
	m[1]  = (T)(0.0); 
	m[2]  = (T)(0.0); 
	m[3]  = (T)(0.0);

	m[4]  = (T)(0.0);
	m[5]  = (T)(1.0);
	m[6]  = (T)(0.0);
	m[7]  = (T)(0.0);

	m[8]  = (T)(0.0);
	m[9]  = (T)(0.0);
	m[10] = (T)(1.0);
	m[11] = (T)(0.0);

	m[12] = (T)(0.0);
	m[13] = (T)(0.0);
	m[14] = (T)(0.0);
	m[15] = (T)(1.0);
}
template<class T>
static void SetTranslate(T m[16], const Vec3 &t)
{
	m[3]  = t[0];
	m[7]  = t[1];
	m[11] = t[2];
}


/*!
 * lƃxNg̐ώZ
 *  - v' = q*v*q(star)
 * @param[in] 
 * @return 
 */
inline Vec3 MultiQV(const dQuaternion q, const Vec3 &v)
{
	Vec3 vd;
	double v_coef = q[0]*q[0]-q[1]*q[1]-q[2]*q[2]-q[3]*q[3];                     
	double u_coef = 2.0*(v[0]*q[1]+v[1]*q[2]+v[2]*q[3]);  
	double c_coef = 2.0*q[0];                                       

	vd[0] = v_coef*v[0]+u_coef*q[1]+c_coef*(q[2]*v[2]-q[3]*v[1]);
	vd[1] = v_coef*v[1]+u_coef*q[2]+c_coef*(q[3]*v[0]-q[1]*v[2]);
	vd[2] = v_coef*v[2]+u_coef*q[3]+c_coef*(q[1]*v[1]-q[2]*v[0]);

	return vd;
}

inline void ODE_GetQuaternionD(dBodyID body, dQuaternion q)
{
	q[0] = dBodyGetQuaternion(body)[0];
	q[1] = dBodyGetQuaternion(body)[1];
	q[2] = dBodyGetQuaternion(body)[2];
	q[3] = dBodyGetQuaternion(body)[3];
}

#ifndef RX_EPSILON
#define RX_EPSILON     (double)(10e-6)
#endif

#define equivalent(a,b)     (((a < b + RX_EPSILON) && (a > b - RX_EPSILON)) ? true : false)

static void CalQuaternion(double &ang, Vec3 &axis, const Vec3 & rotateFrom, const Vec3 & rotateTo)
{
	Vec3 p1, p2;
	double alpha;

	p1 = rotateFrom; 
	unitize(p1);
	p2 = rotateTo;  
	unitize(p2);

	alpha = dot(p1, p2);

	if(equivalent(alpha, 1.0)){ 
		axis = Vec3(0, 0, 0);
		ang = 1;
		return; 
	}

	// ensures that the anti-parallel case leads to a positive dot
	if(equivalent(alpha, -1.0)){
		Vec3 v;

		if(p1[0] != p1[1] || p1[0] != p1[2]){
			v = Vec3(p1[1], p1[2], p1[0]);
		}
		else{
			v = Vec3(-p1[0], p1[1], p1[2]);
		}

		v -= p1*dot(p1, v);
		unitize(v);

		axis = v;
		ang = 180.0;
		return;
	}

	p1 = cross(p1, p2);  
	unitize(p1);

	axis = p1;
	ang = RX_TO_DEGREES(acos(alpha));
}



//-----------------------------------------------------------------------------
// MARK:`
//-----------------------------------------------------------------------------
static vector<string> g_cbDrawStaticDef(3);

template<typename T>
inline string &operator<<(string &cb, const T &a)
{
	cb += ToString(a);
	return cb;
}

/*!
 * `
 * @param[in] cir_str zobt@
 * @param[in] static_str ÓIȕobt@
 * @param[in] w,h EBhETCY
 */
static void DrawStrings(vector<string> &static_str, int w, int h)
{
	// MRK:PrintD
	glDisable(GL_LIGHTING);
	//glColor3f(0.0, 0.0, 0.0);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	float x0 = 5.0f;
	float y0 = h-20.0f;

	// ʏ㕔ɃX^eBbNȃeLXg
	for(int j = 0; j < (int)static_str.size(); ++j){
		glRasterPos2f(x0, y0);

		int size = (int)static_str[j].size();
		for(int i = 0; i < size; ++i){
			char ic = static_str[j][i];
			glutBitmapCharacter(GLUT_BITMAP_9_BY_15, ic);
		}

		y0 -= 20;
	}

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
}



//-----------------------------------------------------------------------------
// MARK:GL`
//-----------------------------------------------------------------------------
static void DrawCylinder(double rad, double len, int slices)
{
	GLUquadricObj *qobj;
	qobj = gluNewQuadric();

	glPushMatrix();
	glTranslatef(0.0, 0.0, -0.5*len);

	gluQuadricDrawStyle(qobj, GLU_FILL);
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluCylinder(qobj, rad, rad, len, slices, slices);

	glPushMatrix();
	glRotatef(180.0, 1.0, 0.0, 0.0);
	gluDisk(qobj, 0.0, rad, slices, slices);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.0, 0.0, len);
	gluDisk(qobj, 0.0, rad, slices, slices);
	glPopMatrix();

	glPopMatrix();
}

static void DrawCapsule(double rad, double len, int slices)
{
	GLUquadricObj *qobj;
	qobj = gluNewQuadric();

	glPushMatrix();
	glTranslatef(0.0, 0.0, -0.5*len);

	gluQuadricDrawStyle(qobj, GLU_FILL);
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluCylinder(qobj, rad, rad, len, slices, slices);

	glPushMatrix();
	glutSolidSphere(rad, slices, slices);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.0, 0.0, len);
	glutSolidSphere(rad, slices, slices);
	glPopMatrix();

	glPopMatrix();

}



#endif // #ifndef _RX_UTILS_H_